
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * Copyright: Philipp Kolmann (c) 2006
 *            philipp@kolmann.at
 * 
 * Purpose: Enable Vol+/- Buttons on USB Headset
 *
 * inspired by: http://www.fuw.edu.pl/~pliszka/linux-USB/#labtec
 *
 */

/* Changelog:
 *
 * 0.0.1: 2005-01-05
 * 	Initial Release
 *
 */

#define DEBUG 0


#include <alsa/asoundlib.h>
#include <stdio.h>
#include <string.h>

#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <asm/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdint.h>

#include <linux/input.h>


snd_ctl_t *chandle = NULL;
snd_hctl_t *handle = NULL; 

snd_ctl_elem_value_t *eval = NULL;
snd_hctl_elem_t *elem = NULL;

int prepare(char* hw) {

	int err=0;
	int j=0;
	int ctls_num;

	snd_ctl_elem_id_t *e_id=NULL;
   	snd_ctl_elem_info_t *einfo;
	snd_ctl_card_info_t *info;
	snd_ctl_elem_list_t *clist;
	snd_hctl_elem_t *ielem = NULL;

	snd_ctl_open(&chandle, hw, 0);
	err = snd_hctl_open(&handle, hw,0);

	snd_ctl_card_info_malloc(&info);
	memset(info,0,snd_ctl_card_info_sizeof());

	snd_ctl_card_info(chandle, info);
	snd_ctl_elem_list_malloc(&clist);
	memset(clist,0,snd_ctl_elem_list_sizeof());

	snd_ctl_elem_list(chandle,clist);

#if DEBUG
	printf("Card %s:/%s\n",
			snd_ctl_card_info_get_id(info),
			snd_ctl_card_info_get_longname(info));
#endif
	
	ctls_num=snd_ctl_elem_list_get_count(clist);
	snd_ctl_elem_list_free(clist);

	snd_hctl_load(handle);
	ielem=snd_hctl_first_elem(handle);

	for( j=0 ; j<ctls_num ; j++ ) {
		snd_ctl_elem_id_malloc(&e_id);
		snd_hctl_elem_get_id(ielem,e_id);

#if DEBUG
		printf("#%d '%s'%d\n",
				snd_ctl_elem_id_get_numid(e_id),
				snd_ctl_elem_id_get_name(e_id),
				snd_ctl_elem_id_get_index(e_id));
#endif

		if (strcmp("Speaker Playback Volume",
					snd_ctl_elem_id_get_name(e_id)) == 0)
		{
#if DEBUG
			printf("found\n");
#endif

			snd_ctl_elem_info_malloc(&einfo);
			memset(einfo,0,snd_ctl_elem_info_sizeof());
			snd_ctl_elem_info_set_id(einfo,e_id);
			snd_hctl_elem_info(ielem,einfo);
	
			snd_ctl_elem_value_malloc(&eval);
			memset(eval,0,snd_ctl_elem_value_sizeof());
			snd_ctl_elem_value_set_id(eval,e_id);

			elem = ielem;
		}

		ielem=snd_hctl_elem_next(ielem);
	}

	return 0;
}


int update_mixer(int change) {
	int val;

	if (elem == NULL) {
		printf("Error, elem undefined!\n");
		exit(-1);
	}

	if (eval == NULL) {
		printf("Error, eval undefined!\n");
		exit(-2);
	}

	snd_hctl_elem_read(elem,eval);

	val = snd_ctl_elem_value_get_integer(eval, 0);
#if DEBUG
	printf("val0: %d\n", val);
#endif
	snd_ctl_elem_value_set_integer(eval, 0, (val+change));
	
	val = snd_ctl_elem_value_get_integer(eval, 1);
#if DEBUG
	printf("val1: %d\n", val);
#endif
	snd_ctl_elem_value_set_integer(eval, 1, (val+change));

	snd_ctl_elem_write(chandle, eval);
		
	return 0;
}

int destroy(void) {
	snd_ctl_close(chandle);
	snd_hctl_close(handle);

	return 0;
}

int main(int argc, char **argv) {

	char* hw;

	int fd = -1;        /* the file descriptor for the device */
	int yalv;           /* loop counter */
	size_t read_bytes;  /* how many bytes were read */
	struct input_event ev[64]; /* the events (up to 64 at once) */

	/* tmporary hack */
	argc=3;
	argv[1]="/dev/input/event2";
	argv[2]="hw:1";

	if (argc != 3) {
		fprintf(stderr, "usage: %s event-device hw-string - probably "
				"/dev/input/evdev0 hw:1\n", argv[0]);
		exit(1);
	}

	if ((fd = open(argv[1], O_RDONLY)) < 0) {
		perror("evdev open");
		exit(2);
	}
		
	
	hw = argv[2];
	prepare(hw);

	while (1) {
		read_bytes = read(fd, ev, sizeof(struct input_event) * 64);

		if (read_bytes < (int) sizeof(struct input_event)) {
			perror("evtest: short read");
			exit (1);
		}

		for (yalv = 0; yalv < (int) (read_bytes / sizeof(struct
						input_event)); yalv++)
		{
			if(ev[yalv].type==EV_KEY &&
					ev[yalv].code==KEY_VOLUMEUP) {
				if(ev[yalv].value==1)
					/*      printf("UP\n"); */
					update_mixer(1);
			} else if(ev[yalv].type==EV_KEY &&
					ev[yalv].code==KEY_VOLUMEDOWN) {
				if(ev[yalv].value==1)
					/*      printf("DOWN\n"); */
					update_mixer(-1);
#if DEBUG
			} else {
				printf("Event: time %ld.%06ld, type %d, "
						"code %d, value %d\n",
						ev[yalv].time.tv_sec,
						ev[yalv].time.tv_usec, 
						ev[yalv].type,ev[yalv].code,
						ev[yalv].value);
#endif
			}
		}
	}

	close(fd);

	destroy();
	return 0;
}
